unit MannUtls;

interface

uses
    Classes, Crt, SysUtils, vpSysLow, vpUtils, Windows;

const
  MASK_ALL = '`1234567890-=\qwertyuiop[]asdfghjkl;''zxcvbnm,./~!@#$%^&*()_+|QWERTYUIOP{}ASDFGHJKL:"ZXCVBNM<>? ';
  MASK_ALPHA = 'qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM ';
  MASK_NUMERIC = '1234567890';
  MASK_ALPHANUMERIC = MASK_ALPHA + MASK_NUMERIC;

function BoolToStr(B: Boolean; T, F: String): String;
 { returns T if B = true or F if B = false }
function Center(S: String): String;
 { returns a string padded on the left and right with spaces }
procedure ConsoleTitle(S: String);
 { sets the console title - win32 only }
function Elapsed(First, Second: LongInt): LongInt;
 { returns difference in seconds from First to Second - handles midnight roll-over }
procedure FastWrite(S: String; X, Y, Attr: Byte);
 { uses direct writes to the screen }
procedure FindFiles(Dir, Mask: String; Files: TStringList; Sub: Boolean);
 { finds all the files in Dir matching Mask and puts them into Files }
procedure GetTime(var H, M, S: LongInt);
 { returns the current hour, minute and second }
function Input(S, Mask: String; ShowLen, MaxLen, Attr: Byte): String;
 { returns a string - is like readln but with maxlen and text attr, and a char mask }
procedure MannCodeToAnsi(var S: String);
 { translates my colour codes to ANSI sequence }
function nReadKey: Char;
 { returns a keypress - short for Nice ReadKey }
function PadLeft(S: String; Ch: Char; Num: Integer): String;
 { returns S padded on the left with Num Ch chars }
function PadRight(S: String; Ch: Char; Num: Integer): String;
 { returns S padded on the right with Num Ch chars }
function Right(S: String): String;
 { right aligns a string by padding left with spaces }
function SecondsSinceMidnight: LongInt;
 { returns the number of seconds since midnight }
function SecToDHMS(I: LongInt): String;
 { returns a string in the format '0d 0h 0m 0s' }
function SecToHM(I: LongInt): String;
 { returns a string in the format 'hh:mm' }
function SecToHMS(I: LongInt): String;
 { returns a string in the format 'hh:mm:ss' }
function SecToMS(I: LongInt): String;
 { returns a string in the format 'mm:ss' }
function TimeZoneBias: Integer;
 { returns the number of minutes difference between UTC and the current TZ }
function UnixTimeToDateTime(D: Double): Double;
 { returns D as a value compatible with TDateTime }

implementation

function BoolToStr(B: Boolean; T, F: String): String;
begin
     if (B) then
        Result := T
     else
         Result := F;
end;

function Center(S: String): String;
begin
     if (Length(S) < 80) then
     begin
          PadLeft(S, ' ', Length(S) + (80 - Length(S)) div 2);
          PadRight(S, ' ', 80);
     end;
     Result := S;
end;

procedure ConsoleTitle(S: String);
var
   P: Array[0..40] of Char;
   PC: PChar;
begin
     PC := P;
     PC := StrPCopy(PC, S);
     SetConsoleTitle(P);
end;

function Elapsed(First, Second: LongInt): LongInt;
begin
     if (Second < First) then
        Inc(Second, 86400);
     Elapsed := Second - First;
end;

procedure FastWrite(S: String; X, Y, Attr: Byte);
begin
     SysWrtCharStrAtt(@S[1], Length(S), X - 1, Y - 1, Attr);
end;

procedure FindFiles(Dir, Mask: String; Files: TStringList; Sub: Boolean);
var
   nStatus: LongInt;
   SRec: TSearchRec;
begin
     if (Dir[Length(Dir)] <> '\') then
        Dir := Dir + '\';

     nStatus := FindFirst(Dir + Mask, faAnyFile, SRec);
     while (nStatus = 0) do
     begin
          if (SRec.Attr and faDirectory = 0) then
             Files.Add(Dir + SRec.Name);
          nStatus := FindNext(SRec);
     end;
     SysUtils.FindClose(SRec);
     if (Sub) then
     begin
          nStatus := FindFirst(Dir + '*.*', faDirectory, SRec);
          while (nStatus = 0) do
          begin
               if (SRec.Attr and faDirectory <> 0) then
               begin
                    if ((SRec.Name <> '.') and (SRec.Name <> '..')) then
                       FindFiles(Dir + SRec.Name, Mask, Files, Sub);
               end;
               nStatus := FindNext(SRec);
          end;
          SysUtils.FindClose(SRec);
     end;
end;

procedure GetTime(var H, M, S: LongInt);
var
   Time: LongInt;
begin
     Time := SecondsSinceMidnight;
     H := Trunc(Time / 3600);
     Time := Time mod 3600;
     M := Trunc(Time / 60);
     S := Time mod 60;
end;

function Input(S, Mask: String; ShowLen, MaxLen, Attr: Byte): String;
var
   Ch: Char;
   X, Y, tAttr: Byte;
begin
     X := WhereX;
     Y := WhereY;
     tAttr := TextAttr;
     TextAttr := Attr;

     if (Length(S) > MaxLen) then
        S := Copy(S, 1, MaxLen);

     if (Length(S) > ShowLen) then
     begin
          Write(Copy(S, Length(S) - ShowLen + 1, ShowLen));
          GotoXY(X + ShowLen, Y);
     end else
     begin
          Write(PadRight(S, ' ', ShowLen));
          GotoXY(X + Length(S), Y);
     end;

     repeat
           Ch := nReadKey;
           if (Ch = #0) then
           begin
                Ch := nReadKey;
                Continue;
           end else

           if ((Length(S) < MaxLen) and (Pos(Ch, Mask) > 0)) or ((Length(S) > 0) and (Ch = #8)) then
           begin
                if (Ch = #8) then
                begin
                     Delete(S, Length(S), 1);
                     Write(#8#32);
                end else
                    S := S + Ch;

                if (Length(S) >= ShowLen) then
                begin
                     GotoXY(X, Y);
                     Write(Copy(S, Length(S) - ShowLen + 1, ShowLen));
                end else
                    Write(Ch);
           end;
     until (Ch = #13);
     Input := S;

     TextAttr := tAttr;
     WriteLn;
end;

procedure MannCodeToAnsi(var S: String);
var
   Ch: Char;
   Str: String;
begin
     while (Pos('', S) > 0) do
     begin
          Ch := UpCase(S[Pos('', S) + 1]);
          case Ch of
               '0': Str := #27 + '[0;30m';
               '1': Str := #27 + '[0;34m';
               '2': Str := #27 + '[0;32m';
               '3': Str := #27 + '[0;36m';
               '4': Str := #27 + '[0;31m';
               '5': Str := #27 + '[0;35m';
               '6': Str := #27 + '[0;33m';
               '7': Str := #27 + '[0;37m';
               '8': Str := #27 + '[1;30m';
               '9': Str := #27 + '[1;34m';
               'A': Str := #27 + '[1;32m';
               'B': Str := #27 + '[1;36m';
               'C': Str := #27 + '[1;31m';
               'D': Str := #27 + '[1;35m';
               'E': Str := #27 + '[1;33m';
               'F': Str := #27 + '[1;37m';
          else
              Str := '' + Ch;
          end;
          Insert(Str, S, Pos('', S));
          Delete(S, Pos('', S), 2);
     end;
end;

function nReadKey: Char;
begin
     repeat
           SysCtrlSleep(1);
     until KeyPressed;
     nReadKey := ReadKey;
end;

function PadLeft(S: String; Ch: Char; Num: Integer): String;
var
   I: Integer;
begin
     if (Length(S) < Num) then
     begin
          for I := 1 to Num - Length(S) do
              S := Ch + S;
     end else
         S := Copy(S, 1, Num);
     PadLeft := S;
end;

function PadRight(S: String; Ch: Char; Num: Integer): String;
var
   I: Integer;
begin
     while (Length(S) < Num) do
           S := S + Ch;
     S := Copy(S, 1, Num);
     PadRight := S;
end;

function Right(S: String): String;
begin
     Result := PadLeft(S, ' ', 80);
end;

function SecondsSinceMidnight: LongInt;
begin
     SecondsSinceMidnight := GetTimeMSec div 1000;
end;

function SecToDHMS(I: LongInt): String;
var
   D, H, M, S: Integer;
begin
     D := Trunc(I / 86400);
     I := I mod 86400;
     H := Trunc(I / 3600);
     I := I mod 3600;
     M := Trunc(I / 60);
     S := I mod 60;
     SecToDHMS := IntToStr(D) + 'd ' + IntToStr(H) + 'h ' + IntToStr(M) + 'm ' + IntToStr(S) + 's';
end;

function SecToHM(I: LongInt): String;
var
   H, M: Integer;
begin
     H := Trunc(I / 3600);
     I := I mod 3600;
     M := Trunc(I / 60);
     SecToHM := PadLeft(IntToStr(H), '0', 2) + ':' + PadLeft(IntToStr(M), '0', 2);
end;

function SecToHMS(I: LongInt): String;
var
   H, M, S: Integer;
begin
     H := Trunc(I / 3600);
     I := I mod 3600;
     M := Trunc(I / 60);
     S := I mod 60;
     SecToHMS := PadLeft(IntToStr(H), '0', 2) + ':' + PadLeft(IntToStr(M), '0', 2) + ':' + PadLeft(IntToStr(S), '0', 2);
end;

function SecToMS(I: LongInt): String;
var
   M, S: Integer;
begin
     M := Trunc(I / 60);
     S := I mod 60;
     SecToMS := PadLeft(IntToStr(M), '0', 2) + ':' + PadLeft(IntToStr(S), '0', 2);
end;

function TimeZoneBias: Integer;
var
   TZI: TTimeZoneInformation;
   Res: DWord;
begin
     Res := GetTimeZoneInformation(TZI);
     if (Res = TIME_ZONE_ID_STANDARD) then
        Result := TZI.Bias + TZI.StandardBias
     else
     if (Res = TIME_ZONE_ID_DAYLIGHT) then
        Result := TZI.Bias + TZI.DaylightBias
     else
         Result := 0;
end;

function UnixTimeToDateTime(D: Double): Double;
begin
{ A Short Note On Whats Happening Here:
  Unix counts the seconds since 01/01/1970 00:00:00 GMT
  DateTimeToStr expects a number counting the days since 30/12/1899 00:00:00 in your own time zone
  Dividing by 86400 turns "Seconds Since" into "Days Since"
  Adding 365 * 70 makes up for the 70 year difference
  Adding 17 accounts for the leap years in those 70 years
  Adding 2 accounts for the 2 day difference between 30/12 and 01/01
  Subtracting TimeZoneBias / 1440 puts it into correct TZ
}
     Result := (D / 86400) + (365 * 70 + 17 + 2) - (TimeZoneBias / 1440);
end;


end.
